@@ -83,13 +83,13 @@ ask_password__response_cb (GtkDialog  *dialog,
 	default:
 		fr_window_dnd_extraction_finished (data->window, TRUE);
 		if (fr_window_is_batch_mode (data->window))
-			gtk_widget_destroy (GTK_WIDGET (data->window));
+			gtk_window_destroy (GTK_WINDOW (data->window));
 		else
 			fr_window_reset_current_action (data->window);
 		break;
 	}
 
-	gtk_widget_destroy (data->dialog);
+	gtk_window_destroy (GTK_WINDOW (data->dialog));
 }
 
 
@@ -114,9 +114,10 @@ dlg_ask_password__common (FrWindow       *window,
 				     "transient-for", GTK_WINDOW (window),
 				     "modal", TRUE,
 				     "use-header-bar", _gtk_settings_get_dialogs_use_header (),
+				     "title", C_("Window title", "Password"),
 				     NULL);
-	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (data->dialog))),
-			   GET_WIDGET ("password_vbox"));
+	gtk_box_append (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (data->dialog))),
+			GET_WIDGET ("password_vbox"));
 	gtk_dialog_add_buttons (GTK_DIALOG (data->dialog),
 				_GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
 				_("_OK"), GTK_RESPONSE_OK,
@@ -152,10 +153,10 @@ dlg_ask_password__common (FrWindow       *window,
 
 		info_bar = gtk_info_bar_new ();
 		label = gtk_label_new (_("Wrong password."));
-		gtk_container_add (GTK_CONTAINER (gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar))), label);
+		gtk_info_bar_add_child (GTK_INFO_BAR (info_bar), label);
 		gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_ERROR);
-		gtk_box_pack_start (GTK_BOX (GET_WIDGET ("error_box")), info_bar, TRUE, TRUE, 0);
-		gtk_widget_show_all (GET_WIDGET ("error_box"));
+		_gtk_box_pack_start (GTK_BOX (GET_WIDGET ("error_box")), info_bar, TRUE, FALSE);
+		gtk_widget_show (GET_WIDGET ("error_box"));
 
 		_gtk_entry_set_locale_text (GTK_ENTRY (data->password_entry),
 					    fr_window_get_password (window));
@@ -183,7 +184,7 @@ dlg_ask_password__common (FrWindow       *window,
 	else
 		gtk_window_set_title (GTK_WINDOW (data->dialog),
 				      fr_window_batch_get_title (window));
-	gtk_widget_show (data->dialog);
+	gtk_window_present (GTK_WINDOW (data->dialog));
 
 	g_free (message);
 	g_free (filename);
