@@ -24,6 +24,7 @@
 #include <unistd.h>
 #include "file-utils.h"
 #include "fr-init.h"
+#include "fr-file-selector-dialog.h"
 #include "glib-utils.h"
 #include "gtk-utils.h"
 #include "fr-window.h"
@@ -77,7 +78,7 @@ extract_cb (GtkDialog   *dialog,
 
 	/* collect extraction options. */
 
-	data->destination = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (data->dialog));
+	data->destination = fr_file_selector_dialog_get_current_folder (FR_FILE_SELECTOR_DIALOG (data->dialog));
 
 	/* check directory existence. */
 
@@ -115,7 +116,7 @@ create_destination_response_cb (GtkDialog    *dialog,
 		      int           response,
 		      DialogData   *data)
 {
-	gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_window_destroy (GTK_WINDOW (dialog));
 
 	if (response != GTK_RESPONSE_YES) {
 		data->do_not_extract = TRUE;
@@ -138,7 +139,7 @@ extract_cb_possibly_try_to_create_destination_directory (DialogData  *data)
 					   _("Extraction not performed"),
 					   _("Could not create the destination folder: %s."),
 					   error->message);
-		g_signal_connect (GTK_MESSAGE_DIALOG (d), "response", G_CALLBACK (gtk_widget_destroy), NULL);
+		g_signal_connect (GTK_MESSAGE_DIALOG (d), "response", G_CALLBACK (gtk_window_destroy), NULL);
 		gtk_widget_show (d);
 	} else {
 		extract_cb_check_whether_preparing_destination_failed (data);
@@ -151,10 +152,10 @@ extraction_not_performed_cb (GtkDialog    *dialog,
 		      int           response,
 		      DialogData   *data)
 {
-	gtk_widget_destroy (GTK_WIDGET (dialog));
+	gtk_window_destroy (GTK_WINDOW (dialog));
 
 	if (fr_window_is_batch_mode (data->window)) {
-		gtk_widget_destroy (data->dialog);
+		gtk_window_destroy (GTK_WINDOW (data->dialog));
 	}
 }
 
@@ -199,7 +200,7 @@ extract_cb_check_permissions (DialogData  *data)
 					   _("Extraction not performed"),
 					   _("You don’t have the right permissions to extract archives in the folder “%s”"),
 					   utf8_path);
-		g_signal_connect (GTK_MESSAGE_DIALOG (d), "response", G_CALLBACK (gtk_widget_destroy), NULL);
+		g_signal_connect (GTK_MESSAGE_DIALOG (d), "response", G_CALLBACK (gtk_window_destroy), NULL);
 		gtk_widget_show (d);
 	} else {
 		extract_cb_start_extracting (data);
@@ -221,15 +222,15 @@ extract_cb_start_extracting (DialogData *data)
 
 	fr_window_set_extract_default_dir (window, destination);
 
-	skip_newer = ! gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_newer_checkbutton"))) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_newer_checkbutton")));
-	junk_paths = ! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_structure_checkbutton")));
+	skip_newer = ! gtk_check_button_get_inconsistent (GTK_CHECK_BUTTON (GET_WIDGET ("keep_newer_checkbutton"))) && gtk_check_button_get_active (GTK_CHECK_BUTTON (GET_WIDGET ("keep_newer_checkbutton")));
+	junk_paths = ! gtk_check_button_get_active (GTK_CHECK_BUTTON (GET_WIDGET ("keep_structure_checkbutton")));
 
-	if (! gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_newer_checkbutton"))))
+	if (! gtk_check_button_get_inconsistent (GTK_CHECK_BUTTON (GET_WIDGET ("keep_newer_checkbutton"))))
 		g_settings_set_boolean (data->settings, PREF_EXTRACT_SKIP_NEWER, skip_newer);
 	g_settings_set_boolean (data->settings, PREF_EXTRACT_RECREATE_FOLDERS, ! junk_paths);
 
-	selected_files = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("selected_files_radiobutton")));
-	pattern_files = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("file_pattern_radiobutton")));
+	selected_files = gtk_check_button_get_active (GTK_CHECK_BUTTON (GET_WIDGET ("selected_files_radiobutton")));
+	pattern_files = gtk_check_button_get_active (GTK_CHECK_BUTTON (GET_WIDGET ("file_pattern_radiobutton")));
 
 	/* create the file list. */
 
@@ -242,10 +243,10 @@ extract_cb_start_extracting (DialogData *data)
 	else if (pattern_files) {
 		const char *pattern;
 
-		pattern = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("file_pattern_entry")));
+		pattern = gtk_editable_get_text (GTK_EDITABLE (GET_WIDGET ("file_pattern_entry")));
 		file_list = fr_window_get_file_list_pattern (window, pattern);
 		if (file_list == NULL) {
-			gtk_widget_destroy (data->dialog);
+			gtk_window_destroy (GTK_WINDOW (data->dialog));
 			return;
 		}
 	}
@@ -260,7 +261,7 @@ extract_cb_start_extracting (DialogData *data)
 	/* close the dialog. */
 
 	data->destination = NULL; /* do not free when destroying the dialog. */
-	gtk_widget_destroy (data->dialog);
+	gtk_window_destroy (GTK_WINDOW (data->dialog));
 
 	/* extract ! */
 
@@ -283,8 +284,10 @@ file_selector_response_cb (GtkDialog    *dialog,
 		      int           response,
 		      DialogData   *data)
 {
+	pref_util_save_window_geometry (GTK_WINDOW (data->dialog), "Extract");
+
 	if ((response == GTK_RESPONSE_CANCEL) || (response == GTK_RESPONSE_DELETE_EVENT)) {
-		gtk_widget_destroy (data->dialog);
+		gtk_window_destroy (GTK_WINDOW (data->dialog));
 	} else if (response == GTK_RESPONSE_OK) {
 		extract_cb (dialog, data);
 	}
@@ -295,15 +298,16 @@ static void
 files_entry_changed_cb (GtkEditable  *widget,
 			DialogData *data)
 {
-	if (! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("file_pattern_radiobutton"))))
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("file_pattern_radiobutton")), TRUE);
+	if (! gtk_check_button_get_active (GTK_CHECK_BUTTON (GET_WIDGET ("file_pattern_radiobutton"))))
+		gtk_check_button_set_active (GTK_CHECK_BUTTON (GET_WIDGET ("file_pattern_radiobutton")), TRUE);
 }
 
 
 static void
 dlg_extract__common (FrWindow *window,
 	             GList    *selected_files,
-	             char     *base_dir_for_selection)
+	             char     *base_dir_for_selection,
+		     gboolean  all_by_default)
 {
 	DialogData *data;
 
@@ -316,35 +320,33 @@ dlg_extract__common (FrWindow *window,
 	data->do_not_extract = FALSE;
 	data->destination = NULL;
 
-	data->dialog = gtk_file_chooser_dialog_new (C_("Window title", "Extract"),
-						    GTK_WINDOW (data->window),
-						    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-						    _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
-						    _GTK_LABEL_EXTRACT, GTK_RESPONSE_OK,
-						    NULL);
+	data->dialog = fr_file_selector_dialog_new (FR_FILE_SELECTOR_MODE_FOLDER,
+						    C_("Window title", "Extract"),
+						    GTK_WINDOW (data->window));
+
+	gtk_dialog_add_button (GTK_DIALOG (data->dialog), _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL);
+	GtkWidget *button = gtk_dialog_add_button (GTK_DIALOG (data->dialog), _GTK_LABEL_EXTRACT, GTK_RESPONSE_OK);
+	gtk_style_context_add_class (gtk_widget_get_style_context (button), "suggested-action");
 
 	gtk_window_set_default_size (GTK_WINDOW (data->dialog), 530, 510);
-	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (data->dialog), FALSE);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (data->dialog), FALSE);
-	gtk_file_chooser_set_create_folders (GTK_FILE_CHOOSER (data->dialog), TRUE);
 	gtk_dialog_set_default_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK);
 
 	data->builder = gtk_builder_new_from_resource (FILE_ROLLER_RESOURCE_UI_PATH "extract-dialog-options.ui");
-	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (data->dialog), GET_WIDGET ("extra_widget"));
+	fr_file_selector_dialog_set_extra_widget (FR_FILE_SELECTOR_DIALOG (data->dialog), GET_WIDGET ("extra_widget"));
 
 	/* Set widgets data. */
 
-	gtk_file_chooser_set_current_folder_file (GTK_FILE_CHOOSER (data->dialog), fr_window_get_extract_default_dir (window), NULL);
+	fr_file_selector_dialog_set_current_folder (FR_FILE_SELECTOR_DIALOG (data->dialog), fr_window_get_extract_default_dir (window));
 
-	if (data->selected_files != NULL)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("selected_files_radiobutton")), TRUE);
+	if ((data->selected_files != NULL) && !all_by_default)
+		gtk_check_button_set_active (GTK_CHECK_BUTTON (GET_WIDGET ("selected_files_radiobutton")), TRUE);
 	else {
 		gtk_widget_set_sensitive (GET_WIDGET ("selected_files_radiobutton"), FALSE);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("all_files_radiobutton")), TRUE);
+		gtk_check_button_set_active (GTK_CHECK_BUTTON (GET_WIDGET ("all_files_radiobutton")), TRUE);
 	}
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_newer_checkbutton")), g_settings_get_boolean (data->settings, PREF_EXTRACT_SKIP_NEWER));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("keep_structure_checkbutton")), g_settings_get_boolean (data->settings, PREF_EXTRACT_RECREATE_FOLDERS));
+	gtk_check_button_set_active (GTK_CHECK_BUTTON (GET_WIDGET ("keep_newer_checkbutton")), g_settings_get_boolean (data->settings, PREF_EXTRACT_SKIP_NEWER));
+	gtk_check_button_set_active (GTK_CHECK_BUTTON (GET_WIDGET ("keep_structure_checkbutton")), g_settings_get_boolean (data->settings, PREF_EXTRACT_RECREATE_FOLDERS));
 
 	/* Set the signals handlers. */
 
@@ -364,7 +366,7 @@ dlg_extract__common (FrWindow *window,
 	/* Run dialog. */
 
 	gtk_window_set_modal (GTK_WINDOW (data->dialog),TRUE);
-	gtk_widget_show (data->dialog);
+	pref_util_restore_window_geometry (GTK_WINDOW (data->dialog), "Extract");
 }
 
 
@@ -377,7 +379,7 @@ dlg_extract (GtkWidget *widget,
 	char     *base_dir;
 
 	files = fr_window_get_selection (window, FALSE, &base_dir);
-	dlg_extract__common (window, files, base_dir);
+	dlg_extract__common (window, files, base_dir, FALSE);
 }
 
 
@@ -390,5 +392,18 @@ dlg_extract_folder_from_sidebar (GtkWidget *widget,
 	char     *base_dir;
 
 	files = fr_window_get_selection (window, TRUE, &base_dir);
-	dlg_extract__common (window, files, base_dir);
+	dlg_extract__common (window, files, base_dir, FALSE);
+}
+
+
+void
+dlg_extract_all_by_default (GtkWidget *widget,
+			    gpointer   callback_data)
+{
+	FrWindow *window = callback_data;
+	GList    *files;
+	char     *base_dir;
+
+	files = fr_window_get_selection (window, FALSE, &base_dir);
+	dlg_extract__common (window, files, base_dir, TRUE);
 }
