@@ -280,7 +280,7 @@ process_line__add (char     *line,
 	FrCommand *command = FR_COMMAND (data);
 	FrArchive *archive = FR_ARCHIVE (data);
 
-	if ((archive->volume_size > 0) && (strncmp (line, "Creating archive ", 17) == 0)) {
+	if ((archive->volume_size > 0) && (strncmp (line, "Creating archive", 16) == 0)) {
 		char  *volume_filename;
 		GFile *volume_file;
 
@@ -556,6 +556,9 @@ fr_command_7z_handle_error (FrCommand *command,
 		/* ignore warnings */
 		fr_error_clear_gerror (error);
 	}
+	if (error->status == 255) {
+		fr_error_take_gerror (error, g_error_new_literal (FR_ERROR, FR_ERROR_ASK_PASSWORD, ""));
+	}
 	else {
 		GList *scan;
 
@@ -579,6 +582,7 @@ const char *sevenz_mime_types[] = {
 	"application/x-apple-diskimage",
 	"application/x-arj",
 	"application/vnd.ms-cab-compressed",
+	"application/vnd.rar",
 	"application/x-cd-image",
 	"application/x-chrome-extension",
 	/*"application/x-cbr",*/
@@ -694,6 +698,7 @@ fr_command_7z_get_capabilities (FrArchive  *archive,
 	}
 	else if (available_formats_full) {
 		if (_g_mime_type_matches (mime_type, "application/x-rar")
+		    || _g_mime_type_matches (mime_type, "application/vnd.rar")
 		    || _g_mime_type_matches (mime_type, "application/x-cbr"))
 		{
 			/* give priority to rar and unrar that supports RAR files better. */
@@ -736,7 +741,7 @@ static const char *
 fr_command_7z_get_packages (FrArchive  *archive,
 			    const char *mime_type)
 {
-	if (_g_mime_type_matches (mime_type, "application/x-rar"))
+	if (_g_mime_type_matches (mime_type, "application/vnd.rar") || _g_mime_type_matches (mime_type, "application/x-rar"))
 		return FR_PACKAGES ("7zip,7zip-rar");
 	else if (_g_mime_type_matches (mime_type, "application/zip") || _g_mime_type_matches (mime_type, "application/vnd.ms-cab-compressed"))
 		return FR_PACKAGES ("7zip,7zip-full");
