@@ -194,27 +194,39 @@ attribute_field_with_space (char *line)
 }
 
 
-static void
+static gboolean
 parse_name_field (char         *line,
 		  FrCommandRar *rar_comm)
 {
-	char     *name_field;
 	FrFileData *fdata;
+	char       *name_field;
 
+	fr_file_data_free (rar_comm->fdata);
 	rar_comm->fdata = fdata = fr_file_data_new ();
 
 	/* read file name. */
 
 	fdata->encrypted = (line[0] == '*') ? TRUE : FALSE;
 
-	if (rar_comm->rar5)
+	name_field = NULL;
+	if (rar_comm->rar5) {
 		/* rar-5 output adds trailing spaces to short file names :( */
-		name_field = g_strchomp (g_strdup (_g_str_get_last_field (line, attribute_field_with_space (line) ? 9 : 8)));
-	else
+		int field_idx = attribute_field_with_space (line) ? 9 : 8;
+		const char *field = _g_str_get_last_field (line, field_idx);
+		if (field == NULL) {
+			// Sometimes the checksum column is empty (seen for directories).
+			field_idx--;
+			field = _g_str_get_last_field (line, field_idx);
+		}
+		if (field != NULL)
+			name_field = g_strchomp (g_strdup (field));
+	}
+	else {
 		name_field = g_strdup (line + 1);
+	}
 
 	if (name_field == NULL)
-		return;
+		return FALSE;
 
 	if (*name_field == '/') {
 		fdata->full_path = g_strdup (name_field);
@@ -229,6 +241,8 @@ parse_name_field (char         *line,
 	fdata->path = _g_path_remove_level (fdata->full_path);
 
 	g_free (name_field);
+
+	return TRUE;
 }
 
 static gboolean
@@ -281,8 +295,10 @@ process_line (char     *line,
 		return;
 	}
 
-	if (rar_comm->rar4_odd_line || rar_comm->rar5)
-		parse_name_field (line, rar_comm);
+	if (rar_comm->rar4_odd_line || rar_comm->rar5) {
+		if (!parse_name_field (line, rar_comm))
+			return;
+	}
 
 	if (! rar_comm->rar4_odd_line) {
 		FrFileData *fdata;
@@ -315,7 +331,8 @@ process_line (char     *line,
 			fr_file_data_free (rar_comm->fdata);
 			rar_comm->fdata = NULL;
 			rar_comm->rar4_odd_line = TRUE;
-			parse_name_field (line, rar_comm);
+			if (!parse_name_field (line, rar_comm))
+				return;
 		}
 		else {
 			if ((strcmp (ratio_field, "<->") == 0)
@@ -803,6 +820,9 @@ fr_command_rar_finalize (GObject *object)
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (FR_IS_COMMAND_RAR (object));
 
+	FrCommandRar *rar_comm = FR_COMMAND_RAR (object);
+	fr_file_data_free (rar_comm->fdata);
+
 	if (G_OBJECT_CLASS (fr_command_rar_parent_class)->finalize)
 		G_OBJECT_CLASS (fr_command_rar_parent_class)->finalize (object);
 }
@@ -851,4 +871,6 @@ fr_command_rar_init (FrCommandRar *self)
 	base->propPassword                 = TRUE;
 	base->propTest                     = TRUE;
 	base->propListFromFile             = TRUE;
+
+	self->fdata = NULL;
 }
