@@ -24,7 +24,7 @@
 #include <string.h>
 #include <time.h>
 #include <glib.h>
-#include "file-data.h"
+#include "fr-file-data.h"
 #include "file-utils.h"
 #include "glib-utils.h"
 #include "fr-command.h"
@@ -49,7 +49,7 @@ static void
 process_data_line (char     *line,
                    gpointer  data)
 {
-        FileData       *fdata;
+        FrFileData *fdata;
         UnsquashfsData *d = data;
         char          **fields;
         char          **tmfields;
@@ -65,7 +65,7 @@ process_data_line (char     *line,
                 return;
         }
 
-        fdata = file_data_new ();
+        fdata = fr_file_data_new ();
 
         fields = _g_str_split_line (line, 5);
         fdata->size = g_ascii_strtoull (fields[2], NULL, 10);
@@ -112,7 +112,7 @@ process_data_line (char     *line,
         fdata->path = _g_path_remove_level (fdata->full_path);
 
         if (*fdata->name == 0)
-                file_data_free (fdata);
+                fr_file_data_free (fdata);
         else
                 fr_archive_add_file (FR_ARCHIVE (d->command), fdata);
 }
@@ -179,12 +179,12 @@ fr_command_unsquashfs_get_mime_types (FrArchive *archive)
 }
 
 
-static FrArchiveCap
+static FrArchiveCaps
 fr_command_unsquashfs_get_capabilities (FrArchive  *archive,
                                         const char *mime_type,
                                         gboolean    check_command)
 {
-        FrArchiveCap capabilities;
+        FrArchiveCaps capabilities;
 
         capabilities = FR_ARCHIVE_CAN_STORE_MANY_FILES;
         if (_g_program_is_available ("unsquashfs", check_command))
@@ -198,7 +198,7 @@ static const char *
 fr_command_unsquashfs_get_packages (FrArchive  *archive,
                                     const char *mime_type)
 {
-        return PACKAGES ("unsquashfs");
+        return FR_PACKAGES ("unsquashfs");
 }
 
 
