@@ -23,31 +23,13 @@
 #include <string.h>
 #include "gtk-utils.h"
 
-#define LOAD_BUFFER_SIZE 65536
-#define FILE_ROLLER_RESOURCE_UI_PATH "/org/gnome/FileRoller/ui/"
 
-
-static void
-count_selected (GtkTreeModel *model,
-		GtkTreePath  *path,
-		GtkTreeIter  *iter,
-		gpointer      data)
+void
+_gtk_dialog_run (GtkDialog *dialog)
 {
-	int *n = data;
-	*n = *n + 1;
-}
-
-
-int
-_gtk_tree_selection_count_selected (GtkTreeSelection *selection)
-{
-	int n = 0;
-
-	if (selection == NULL)
-		return 0;
-
-	gtk_tree_selection_selected_foreach (selection, count_selected, &n);
-	return n;
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	g_signal_connect (dialog, "response", G_CALLBACK (gtk_window_destroy), NULL);
+	gtk_window_present (GTK_WINDOW (dialog));
 }
 
 
@@ -101,21 +83,8 @@ _gtk_message_dialog_new (GtkWindow      *parent,
 }
 
 
-static GtkWidget *
-create_button (const char *text)
-{
-	GtkWidget *button;
-
-	button = gtk_button_new_with_mnemonic (text);
-	gtk_widget_set_can_default (button, TRUE);
-	gtk_widget_show (button);
-
-	return button;
-}
-
-
-char *
-_gtk_request_dialog_run (GtkWindow      *parent,
+GtkWidget *
+_gtk_request_dialog_new (GtkWindow      *parent,
 			 GtkDialogFlags  flags,
 			 const char     *title,
 			 const char     *message,
@@ -128,11 +97,9 @@ _gtk_request_dialog_run (GtkWindow      *parent,
 	GtkWidget  *dialog;
 	GtkWidget  *label;
 	GtkWidget  *entry;
-	GtkWidget  *content_area;
 	GtkWidget  *request_box;
-	char       *result;
 
-	builder = _gtk_builder_new_from_resource ("request-dialog.ui");
+	builder = gtk_builder_new_from_resource (FILE_ROLLER_RESOURCE_UI_PATH "request-dialog.ui");
 	request_box = _gtk_builder_get_widget (builder, "request_box");
 
 	dialog = g_object_new (GTK_TYPE_DIALOG,
@@ -143,9 +110,8 @@ _gtk_request_dialog_run (GtkWindow      *parent,
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), (flags & GTK_DIALOG_DESTROY_WITH_PARENT));
 	gtk_window_set_title (GTK_WINDOW (dialog), title);
-	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_container_add (GTK_CONTAINER (content_area), request_box);
-	g_object_weak_ref (G_OBJECT (request_box), (GWeakNotify) g_object_unref, builder);
+	gtk_box_append (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), request_box);
+	g_object_set_data_full (G_OBJECT (dialog), "builder", builder, g_object_unref);
 
 	if (flags & GTK_DIALOG_MODAL)
 		_gtk_dialog_add_to_window_group (GTK_DIALOG (dialog));
@@ -155,30 +121,38 @@ _gtk_request_dialog_run (GtkWindow      *parent,
 
 	entry = _gtk_builder_get_widget (builder, "value_entry");
 	gtk_entry_set_max_length (GTK_ENTRY (entry), max_length);
-	gtk_entry_set_text (GTK_ENTRY (entry), default_value);
+	gtk_editable_set_text (GTK_EDITABLE (entry), default_value);
+	gtk_widget_grab_focus (entry);
 
 	/* Add buttons */
 
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
-				      create_button (no_button_text),
+				      gtk_button_new_with_mnemonic (no_button_text),
 				      GTK_RESPONSE_CANCEL);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
-				      create_button (yes_button_text),
+				      gtk_button_new_with_mnemonic (yes_button_text),
 				      GTK_RESPONSE_YES);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
-	/* Run dialog */
+	return dialog;
+}
 
-	gtk_widget_grab_focus (entry);
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
-		result = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-	else
-		result = NULL;
+char *
+_gth_request_dialog_get_text (GtkDialog *dialog)
+{
+	GtkBuilder *builder;
+	GtkWidget  *entry;
 
-	gtk_widget_destroy (dialog);
+	builder = g_object_get_data (G_OBJECT (dialog), "builder");
+	if (builder == NULL)
+		return NULL;
 
-	return result;
+	entry = _gtk_builder_get_widget (builder, "value_entry");
+	if (entry == NULL)
+		return NULL;
+
+	return g_strdup (gtk_editable_get_text (GTK_EDITABLE (entry)));
 }
 
 
@@ -229,22 +203,24 @@ _gtk_error_dialog_new (GtkWindow      *parent,
 		GList         *scan;
 
 		output_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
-		gtk_box_pack_end (GTK_BOX (gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog))),
-				  output_box, TRUE, TRUE, 0);
+		_gtk_box_pack_end (GTK_BOX (gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog))),
+				   output_box,
+				   TRUE,
+				   FALSE);
 
 		label = gtk_label_new_with_mnemonic (_("C_ommand Line Output:"));
-		gtk_box_pack_start (GTK_BOX (output_box), label, FALSE, FALSE, 0);
+		gtk_box_append (GTK_BOX (output_box), label);
 
 		scrolled_window = g_object_new (GTK_TYPE_SCROLLED_WINDOW,
-						"shadow-type", GTK_SHADOW_IN,
+						"has-frame", TRUE,
 						"width-request", 450,
 						"height-request", 200,
 						NULL);
-		gtk_box_pack_start (GTK_BOX (output_box), scrolled_window, TRUE, TRUE, 0);
+		_gtk_box_pack_start (GTK_BOX (output_box), scrolled_window, TRUE, TRUE);
 
 		text_view = gtk_text_view_new ();
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), text_view);
-		gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
+		gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window), text_view);
 
 		text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
 		gtk_text_buffer_create_tag (text_buffer, "monospace",
@@ -267,7 +243,7 @@ _gtk_error_dialog_new (GtkWindow      *parent,
 			gtk_text_buffer_insert (text_buffer, &iter, "\n", 1);
 		}
 
-		gtk_widget_show_all (output_box);
+		gtk_widget_show (output_box);
 	}
 
 	return dialog;
@@ -294,25 +270,21 @@ _gtk_error_dialog_run (GtkWindow  *parent,
 				      message,
 				      _GTK_LABEL_CLOSE, GTK_RESPONSE_CANCEL,
 				      NULL);
+	_gtk_dialog_run (GTK_DIALOG (d));
+
 	g_free (message);
-
-	g_signal_connect (G_OBJECT (d), "response",
-			  G_CALLBACK (gtk_widget_destroy),
-			  NULL);
-
-	gtk_widget_show (d);
 }
 
 
 void
 _gtk_dialog_add_to_window_group (GtkDialog *dialog)
 {
-	GtkWidget *toplevel;
+	GtkRoot *toplevel;
 
 	g_return_if_fail (dialog != NULL);
 
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (dialog));
-	if (gtk_widget_is_toplevel (toplevel) && gtk_window_has_group (GTK_WINDOW (toplevel)))
+	toplevel = gtk_widget_get_root (GTK_WIDGET (dialog));
+	if ((toplevel != NULL) && gtk_window_has_group (GTK_WINDOW (toplevel)))
 		gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (toplevel)), GTK_WINDOW (dialog));
 }
 
@@ -327,10 +299,7 @@ _gtk_entry_set_locale_text (GtkEntry   *entry,
 		return;
 
 	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
-	if (utf8_text != NULL)
-		gtk_entry_set_text (entry, utf8_text);
-	else
-		gtk_entry_set_text (entry, "");
+	gtk_editable_set_text (GTK_EDITABLE (entry), (utf8_text != NULL) ? utf8_text : "");
 	g_free (utf8_text);
 }
 
@@ -341,7 +310,7 @@ _gtk_entry_get_locale_text (GtkEntry *entry)
 	const char *utf8_text;
 	char       *text;
 
-	utf8_text = gtk_entry_get_text (entry);
+	utf8_text = gtk_editable_get_text (GTK_EDITABLE (entry));
 	if (utf8_text == NULL)
 		return NULL;
 
@@ -351,220 +320,19 @@ _gtk_entry_get_locale_text (GtkEntry *entry)
 }
 
 
-void
-_gtk_label_set_locale_text (GtkLabel   *label,
-			    const char *text)
-{
-	char *utf8_text;
-
-	utf8_text = g_locale_to_utf8 (text, -1, NULL, NULL, NULL);
-	if (utf8_text != NULL) {
-		gtk_label_set_text (label, utf8_text);
-		g_free (utf8_text);
-	}
-	else
-		gtk_label_set_text (label, "");
-}
-
-
-char *
-_gtk_label_get_locale_text (GtkLabel *label)
-{
-	const char *utf8_text;
-	char       *text;
-
-	utf8_text = gtk_label_get_text (label);
-	if (utf8_text == NULL)
-		return NULL;
-
-	text = g_locale_from_utf8 (utf8_text, -1, NULL, NULL, NULL);
-
-	return text;
-}
-
-
-void
-_gtk_entry_set_filename_text (GtkEntry   *entry,
-			      const char *text)
-{
-	char *utf8_text;
-
-	utf8_text = g_filename_to_utf8 (text, -1, NULL, NULL, NULL);
-	if (utf8_text != NULL) {
-		gtk_entry_set_text (entry, utf8_text);
-		g_free (utf8_text);
-	}
-	else
-		gtk_entry_set_text (entry, "");
-}
-
-
-char *
-_gtk_entry_get_filename_text (GtkEntry   *entry)
-{
-	const char *utf8_text;
-	char       *text;
-
-	utf8_text = gtk_entry_get_text (entry);
-	if (utf8_text == NULL)
-		return NULL;
-
-	text = g_filename_from_utf8 (utf8_text, -1, NULL, NULL, NULL);
-
-	return text;
-}
-
-
-void
-_gtk_label_set_filename_text (GtkLabel   *label,
-			      const char *text)
-{
-	char *utf8_text;
-
-	utf8_text = g_filename_display_name (text);
-	gtk_label_set_text (label, utf8_text);
-	g_free (utf8_text);
-}
-
-
-char *
-_gtk_label_get_filename_text (GtkLabel   *label)
-{
-	const char *utf8_text;
-	char       *text;
-
-	utf8_text = gtk_label_get_text (label);
-	if (utf8_text == NULL)
-		return NULL;
-
-	text = g_filename_from_utf8 (utf8_text, -1, NULL, NULL, NULL);
-
-	return text;
-}
-
-
-GdkPixbuf *
-_g_icon_get_pixbuf (GIcon        *icon,
-		    int           icon_size,
-		    GtkIconTheme *icon_theme)
-{
-	GdkPixbuf   *pixbuf = NULL;
-	GtkIconInfo *icon_info;
-
-	icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme,
-						    icon,
-						    icon_size,
-						    GTK_ICON_LOOKUP_USE_BUILTIN);
-
-	if (icon_info != NULL) {
-		GError *error = NULL;
-
-		pixbuf = gtk_icon_info_load_icon (icon_info, &error);
-		if (error != NULL) {
-			g_print ("%s\n", error->message);
-			g_error_free (error);
-		}
-
-		g_object_unref (icon_info);
-	}
-
-	return pixbuf;
-}
-
-
-GdkPixbuf *
-_g_mime_type_get_icon (const char   *mime_type,
-		       int           icon_size,
-		       GtkIconTheme *icon_theme)
-{
-	GdkPixbuf *pixbuf = NULL;
-	GIcon     *icon;
-
-	if (icon_theme == NULL)
-		icon_theme = gtk_icon_theme_get_default ();
-
-	icon = g_content_type_get_icon (mime_type);
-	if (icon == NULL)
-		icon = g_themed_icon_new ("text-x-generic");
-	pixbuf = _g_icon_get_pixbuf (icon, icon_size, icon_theme);
-
-	g_object_unref (icon);
-
-	return pixbuf;
-}
-
-
 void
 _gtk_show_help_dialog (GtkWindow  *parent,
 		       const char *section)
 {
-	char   *uri;
-	GError *error = NULL;
+	char *uri;
 
 	uri = g_strconcat ("help:file-roller", section ? "?" : NULL, section, NULL);
-	if (! gtk_show_uri_on_window (parent, uri, GDK_CURRENT_TIME, &error)) {
-  		GtkWidget *dialog;
+	gtk_show_uri (parent, uri, GDK_CURRENT_TIME);
 
-		dialog = _gtk_message_dialog_new (parent,
-						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  _("Could not display help"),
-						  error->message,
-						  _GTK_LABEL_CLOSE, GTK_RESPONSE_OK,
-						  NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-
-		gtk_widget_show (dialog);
-
-		g_clear_error (&error);
-	}
 	g_free (uri);
 }
 
 
-GtkBuilder *
-_gtk_builder_new_from_file (const char *ui_file)
-{
-	char       *filename;
-	GtkBuilder *builder;
-	GError     *error = NULL;
-
-	filename = g_build_filename (UIDIR, ui_file, NULL);
-	builder = gtk_builder_new ();
-        if (! gtk_builder_add_from_file (builder, filename, &error)) {
-                g_warning ("%s\n", error->message);
-                g_clear_error (&error);
-        }
-	g_free (filename);
-
-        return builder;
-}
-
-
-GtkBuilder *
-_gtk_builder_new_from_resource (const char *resource_path)
-{
-	GtkBuilder *builder;
-	char       *full_path;
-	GError     *error = NULL;
-
-	builder = gtk_builder_new ();
-	full_path = g_strconcat (FILE_ROLLER_RESOURCE_UI_PATH, resource_path, NULL);
-        if (! gtk_builder_add_from_resource (builder, full_path, &error)) {
-                g_warning ("%s\n", error->message);
-                g_clear_error (&error);
-        }
-	g_free (full_path);
-
-        return builder;
-}
-
-
 GtkWidget *
 _gtk_builder_get_widget (GtkBuilder *builder,
 			 const char *name)
@@ -573,26 +341,19 @@ _gtk_builder_get_widget (GtkBuilder *builder,
 }
 
 
-int
-_gtk_widget_lookup_for_size (GtkWidget   *widget,
-			     GtkIconSize  icon_size)
-{
-	int w, h;
-
-	if (! gtk_icon_size_lookup (icon_size, &w, &h))
-		w = h = 16;
-
-	return MAX (w, h);
-}
-
-
 static void
 password_entry_icon_press_cb (GtkEntry            *entry,
 			      GtkEntryIconPosition icon_pos,
-			      GdkEvent            *event,
 			      gpointer             user_data)
 {
-	gtk_entry_set_visibility (entry, ! gtk_entry_get_visibility (entry));
+	gboolean visibility;
+
+	visibility = gtk_entry_get_visibility (entry);
+	gtk_entry_set_visibility (entry, ! visibility);
+	if (visibility)
+		gtk_entry_set_icon_from_icon_name (entry, GTK_ENTRY_ICON_SECONDARY, "view-reveal-symbolic");
+	else
+		gtk_entry_set_icon_from_icon_name (entry, GTK_ENTRY_ICON_SECONDARY, "view-conceal-symbolic");
 }
 
 
@@ -600,11 +361,11 @@ void
 _gtk_entry_use_as_password_entry (GtkEntry *entry)
 {
 	gtk_entry_set_visibility (entry, FALSE);
-	gtk_entry_set_icon_from_icon_name (entry, GTK_ENTRY_ICON_SECONDARY, "security-medium-symbolic");
+	gtk_entry_set_icon_from_icon_name (entry, GTK_ENTRY_ICON_SECONDARY, "view-reveal-symbolic");
 	gtk_entry_set_icon_activatable (entry, GTK_ENTRY_ICON_SECONDARY, TRUE);
 	gtk_entry_set_icon_tooltip_text (entry, GTK_ENTRY_ICON_SECONDARY, _("Change password visibility"));
 
-	g_signal_connect (entry,
+	g_signal_connect (GTK_ENTRY (entry),
 			  "icon-press",
 			  G_CALLBACK (password_entry_icon_press_cb),
 			  NULL);
@@ -640,36 +401,13 @@ _gtk_image_button_new_for_header_bar (const char *icon_name)
 {
 	GtkWidget *button;
 
-	button = gtk_button_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+	button = gtk_button_new_from_icon_name (icon_name);
 	_gtk_menu_button_set_style_for_header_bar (button);
 
 	return button;
 }
 
 
-GtkWidget *
-_gtk_header_bar_create_text_button (const char       *label,
-				    const char       *tooltip,
-				    const char       *action_name)
-{
-	GtkWidget *button;
-
-	g_return_val_if_fail (label != NULL, NULL);
-	g_return_val_if_fail (action_name != NULL, NULL);
-
-	button = gtk_button_new_with_label (label);
-	gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
-	gtk_style_context_add_class (gtk_widget_get_style_context (button), "text-button");
-	gtk_style_context_remove_class (gtk_widget_get_style_context (button), "image-button");
-	gtk_actionable_set_action_name (GTK_ACTIONABLE (button), action_name);
-	if (tooltip != NULL)
-		gtk_widget_set_tooltip_text (button, tooltip);
-	gtk_widget_show (button);
-
-	return button;
-}
-
-
 GtkWidget *
 _gtk_header_bar_create_image_button (const char       *icon_name,
 				     const char       *tooltip,
@@ -701,12 +439,12 @@ _gtk_header_bar_create_image_toggle_button (const char       *icon_name,
 	g_return_val_if_fail (action_name != NULL, NULL);
 
 	button = gtk_toggle_button_new ();
-	gtk_container_add (GTK_CONTAINER (button), gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU));
+	gtk_button_set_icon_name (GTK_BUTTON (button), icon_name);
 	_gtk_menu_button_set_style_for_header_bar (button);
 	gtk_actionable_set_action_name (GTK_ACTIONABLE (button), action_name);
 	if (tooltip != NULL)
 		gtk_widget_set_tooltip_text (button, tooltip);
-	gtk_widget_show_all (button);
+	gtk_widget_show (button);
 
 	return button;
 }
@@ -715,95 +453,33 @@ _gtk_header_bar_create_image_toggle_button (const char       *icon_name,
 /* -- _gtk_window_add_accelerator_for_action -- */
 
 
-typedef struct {
-	GtkWindow *window;
-	char      *action_name;
-	GVariant  *target;
-} AccelData;
-
-
-static void
-accel_data_free (gpointer  user_data,
-                 GClosure *closure)
-{
-	AccelData *accel_data = user_data;
-
-	g_return_if_fail (accel_data != NULL);
-
-	if (accel_data->target != NULL)
-		g_variant_unref (accel_data->target);
-	g_free (accel_data->action_name);
-	g_free (accel_data);
-}
-
-
-static gboolean
-window_accelerator_activated_cb (GtkAccelGroup	*accel_group,
-				 GObject		*object,
-				 guint		 key,
-				 GdkModifierType	 mod,
-				 gpointer		 user_data)
-{
-	AccelData *accel_data = user_data;
-	GAction   *action;
-
-	action = g_action_map_lookup_action (G_ACTION_MAP (accel_data->window), accel_data->action_name);
-	if (action != NULL) {
-		g_action_activate (action, accel_data->target);
-		return GDK_EVENT_STOP;
-	}
-
-	return GDK_EVENT_PROPAGATE;
-}
-
-
 void
-_gtk_window_add_accelerator_for_action (GtkWindow	*window,
-					GtkAccelGroup	*accel_group,
-					const char	*action_name,
+_gtk_add_accelerator_for_action (const char	*action_name,
 					const char	*accel,
 					GVariant	*target)
 {
-	AccelData	*accel_data;
-	guint		 key;
-	GdkModifierType  mods;
-	GClosure	*closure;
+	g_autofree gchar *detailed_action = NULL;
 
 	if ((action_name == NULL) || (accel == NULL))
 		return;
 
-	if (g_str_has_prefix (action_name, "app."))
-		return;
+	if (target == NULL) {
+		detailed_action = g_strdup(action_name);
+	} else {
+		g_autofree gchar *target_string = g_variant_print (target, TRUE);
+		detailed_action = g_strconcat (action_name, "(", target_string, ")", NULL);
+	}
 
-	accel_data = g_new0 (AccelData, 1);
-	accel_data->window = window;
-	/* remove the win. prefix from the action name */
-	if (g_str_has_prefix (action_name, "win."))
-		accel_data->action_name = g_strdup (action_name + strlen ("win."));
-	else
-		accel_data->action_name = g_strdup (action_name);
-	if (target != NULL)
-		accel_data->target = g_variant_ref (target);
+	_gtk_application_add_accelerator_for_action (GTK_APPLICATION (g_application_get_default ()), detailed_action, accel);
 
-	gtk_accelerator_parse (accel, &key, &mods);
-	closure = g_cclosure_new (G_CALLBACK (window_accelerator_activated_cb),
-				  accel_data,
-				  accel_data_free);
-	gtk_accel_group_connect (accel_group,
-				 key,
-				 mods,
-				 0,
-				 closure);
 }
 
 
-/* -- _gtk_window_add_accelerators_from_menu --  */
+/* -- _gtk_add_accelerators_from_menu --  */
 
 
 static void
-add_accelerators_from_menu_item (GtkWindow      *window,
-				 GtkAccelGroup  *accel_group,
-				 GMenuModel     *model,
+add_accelerators_from_menu_item (GMenuModel     *model,
 				 int             item)
 {
 	GMenuAttributeIter	*iter;
@@ -813,6 +489,7 @@ add_accelerators_from_menu_item (GtkWindow      *window,
 	const char		*action = NULL;
 	GVariant		*target = NULL;
 
+
 	iter = g_menu_model_iterate_item_attributes (model, item);
 	while (g_menu_attribute_iter_get_next (iter, &key, &value)) {
 		if (g_str_equal (key, "action") && g_variant_is_of_type (value, G_VARIANT_TYPE_STRING))
@@ -825,21 +502,15 @@ add_accelerators_from_menu_item (GtkWindow      *window,
 	}
 	g_object_unref (iter);
 
-	_gtk_window_add_accelerator_for_action (window,
-						accel_group,
-						action,
-						accel,
-						target);
+	_gtk_add_accelerator_for_action (action, accel, target);
 
 	if (target != NULL)
 		g_variant_unref (target);
 }
 
 
-static void
-add_accelerators_from_menu (GtkWindow      *window,
-			    GtkAccelGroup  *accel_group,
-			    GMenuModel     *model)
+void
+_gtk_add_accelerators_from_menu (GMenuModel *model)
 {
 	int		 i;
 	GMenuLinkIter	*iter;
@@ -847,11 +518,11 @@ add_accelerators_from_menu (GtkWindow      *window,
 	GMenuModel	*m;
 
 	for (i = 0; i < g_menu_model_get_n_items (model); i++) {
-		add_accelerators_from_menu_item (window, accel_group, model, i);
+		add_accelerators_from_menu_item (model, i);
 
 		iter = g_menu_model_iterate_item_links (model, i);
 		while (g_menu_link_iter_get_next (iter, &key, &m)) {
-			add_accelerators_from_menu (window, accel_group, m);
+			_gtk_add_accelerators_from_menu (m);
 			g_object_unref (m);
 		}
 		g_object_unref (iter);
@@ -859,18 +530,6 @@ add_accelerators_from_menu (GtkWindow      *window,
 }
 
 
-void
-_gtk_window_add_accelerators_from_menu (GtkWindow  *window,
-					GMenuModel *menu)
-{
-	GtkAccelGroup *accel_group;
-
-	accel_group = gtk_accel_group_new ();
-	add_accelerators_from_menu (window, accel_group, menu);
-	gtk_window_add_accel_group (window, accel_group);
-}
-
-
 gboolean
 _gtk_settings_get_dialogs_use_header (void)
 {
@@ -911,3 +570,224 @@ _gtk_application_add_accelerators (GtkApplication *app,
 							     acc->accelerator);
 	}
 }
+
+void
+_gtk_popover_popup_at_selected (GtkPopover *popover, GtkTreeView *tree_view)
+{
+	int width = gtk_widget_get_allocated_width (GTK_WIDGET (tree_view));
+	int height = gtk_widget_get_allocated_height (GTK_WIDGET (tree_view));
+	gdouble center_x = 0.5 * width;
+	gdouble center_y = 0.5 * height;
+
+	GdkRectangle rect;
+	GtkTreeSelection *selection;
+	GList *list;
+
+	selection = gtk_tree_view_get_selection (tree_view);
+	list = gtk_tree_selection_get_selected_rows (selection, NULL);
+	if (list) {
+		int min_y = height;
+		int max_y = 0;
+		int max_height = 0;
+
+		for (GList *item = list; item; item = item->next) {
+			GtkTreePath *path = list->data;
+
+			gtk_tree_view_get_cell_area (tree_view, path, NULL, &rect);
+			gtk_tree_view_convert_bin_window_to_widget_coords (tree_view, rect.x, rect.y, &rect.x, &rect.y);
+
+			min_y = MIN (min_y, rect.y);
+			if (rect.y > max_y) {
+				max_y = rect.y;
+				max_height = rect.height;
+			}
+		}
+
+		rect.x = CLAMP (center_x - 20, 0, width - 40);
+		rect.y = min_y;
+		rect.width = 40;
+		rect.height = max_y + max_height - min_y;
+
+		g_list_free_full (list, (GDestroyNotify) gtk_tree_path_free);
+	} else {
+		rect.x = center_x;
+		rect.y = center_y;
+		rect.width = 1;
+		rect.height = 1;
+	}
+
+	gtk_popover_set_pointing_to (GTK_POPOVER (popover), &rect);
+	gtk_popover_popup (GTK_POPOVER (popover));
+}
+
+void
+_gtk_popover_popup_at_position (GtkPopover *popover, gdouble x, gdouble y)
+{
+	GdkRectangle rect;
+
+	rect.x = x;
+	rect.y = y;
+	rect.width = 1;
+	rect.height = 1;
+
+	gtk_popover_set_pointing_to (GTK_POPOVER (popover), &rect);
+	gtk_popover_popup (GTK_POPOVER (popover));
+}
+
+
+void
+_gtk_box_pack_start (GtkBox    *box,
+		     GtkWidget *child,
+		     gboolean   hexpand,
+		     gboolean   vexpand)
+{
+	gtk_widget_set_hexpand (child, hexpand);
+	gtk_widget_set_vexpand (child, vexpand);
+	gtk_box_prepend (box, child);
+}
+
+
+void
+_gtk_box_pack_end (GtkBox    *box,
+		   GtkWidget *child,
+		   gboolean   hexpand,
+		   gboolean   vexpand)
+{
+	gtk_widget_set_hexpand (child, hexpand);
+	gtk_widget_set_vexpand (child, vexpand);
+	gtk_box_append (box, child);
+}
+
+
+void
+_gtk_widget_set_margin (GtkWidget *widget, int margin)
+{
+	gtk_widget_set_margin_start (widget, margin);
+	gtk_widget_set_margin_end (widget, margin);
+	gtk_widget_set_margin_top (widget, margin);
+	gtk_widget_set_margin_bottom (widget, margin);
+}
+
+
+void
+_gtk_show_folder (GtkWindow *parent_window,
+		  GFile     *folder)
+{
+	char *uri = g_file_get_uri (folder);
+	gtk_show_uri (parent_window, uri, GDK_CURRENT_TIME);
+	g_free (uri);
+}
+
+
+typedef struct {
+	GtkWindow *window;
+	GFile     *folder;
+} OpenFolderData;
+
+
+static OpenFolderData *
+open_folder_data_new (GtkWindow *window,
+		      GFile     *folder)
+{
+	OpenFolderData *data;
+
+	data = g_new (OpenFolderData, 1);
+	data->window = g_object_ref (window);
+	data->folder = g_object_ref (folder);
+
+	return data;
+}
+
+
+static void
+open_folder_data_free (OpenFolderData *data)
+{
+	g_object_unref (data->window);
+	g_object_unref (data->folder);
+	g_free (data);
+}
+
+
+static void
+file_manager_show_items_cb (GObject      *source_object,
+			    GAsyncResult *res,
+			    gpointer      user_data)
+{
+	OpenFolderData *data = user_data;
+	GDBusProxy     *proxy;
+	GVariant       *values;
+	GError         *error = NULL;
+
+	proxy = G_DBUS_PROXY (source_object);
+	values = g_dbus_proxy_call_finish (proxy, res, &error);
+	if (values == NULL) {
+		_gtk_show_folder (data->window, data->folder);
+		g_clear_error (&error);
+	}
+
+	if (values != NULL)
+		g_variant_unref (values);
+	g_object_unref (proxy);
+	open_folder_data_free (data);
+}
+
+
+void
+_gtk_show_file_in_container (GtkWindow *parent_window,
+			     GFile     *file)
+{
+	GFile           *parent;
+	GDBusConnection *connection;
+	GError          *error = NULL;
+
+	parent = g_file_get_parent (file);
+	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+	if (connection != NULL) {
+		GDBusProxy *proxy;
+
+		proxy = g_dbus_proxy_new_sync (connection,
+					       G_DBUS_PROXY_FLAGS_NONE,
+					       NULL,
+					       "org.freedesktop.FileManager1",
+					       "/org/freedesktop/FileManager1",
+					       "org.freedesktop.FileManager1",
+					       NULL,
+					       &error);
+
+		if (proxy != NULL) {
+			static int   sequence = 0;
+			char       **uris;
+			char        *startup_id;
+
+			uris = g_new (char *, 2);
+			uris[0] = g_file_get_uri (file);
+			uris[1] = NULL;
+
+			startup_id = g_strdup_printf ("%s-%lu-%s-%s-%d_TIME%lu",
+						      g_get_prgname (),
+						      (unsigned long) getpid (),
+						      g_get_host_name (),
+						      "org.freedesktop.FileManager1",
+						      sequence++,
+						      (unsigned long) g_get_real_time ());
+
+			g_dbus_proxy_call (proxy,
+					   "ShowItems",
+					   g_variant_new ("(^ass)", uris, startup_id),
+					   G_DBUS_CALL_FLAGS_NONE,
+					   G_MAXINT,
+					   NULL,
+					   file_manager_show_items_cb,
+					   open_folder_data_new (parent_window, parent));
+
+			g_free (startup_id);
+			g_strfreev (uris);
+
+			return;
+		}
+	}
+
+	_gtk_show_folder (parent_window, parent);
+
+	g_object_unref (parent);
+}
